@@ -91,15 +91,7 @@ void ExternalSorter::Record::loadFromFile(Tools::TemporaryFile& f)
 	m_id = static_cast<id_type>(f.readUInt64());
 	uint32_t dim = f.readUInt32();
 	m_s = f.readUInt32();
-
-	if (dim != m_r.m_dimension)
-	{
-		delete[] m_r.m_pLow;
-		delete[] m_r.m_pHigh;
-		m_r.m_dimension = dim;
-		m_r.m_pLow = new double[dim];
-		m_r.m_pHigh = new double[dim];
-	}
+	m_r.makeDimension(dim);
 
 	for (uint32_t i = 0; i < m_r.m_dimension; ++i)
 	{
@@ -333,10 +325,6 @@ void BulkLoader::bulkLoadUsingSTR(
 	NodePtr n = pTree->readNode(pTree->m_rootID);
 	pTree->deleteNode(n.get());
 
-	#ifndef NDEBUG
-	std::cerr << "RTree::BulkLoader: Sorting data." << std::endl;
-	#endif
-
     std::shared_ptr<ExternalSorter> es = std::shared_ptr<ExternalSorter>(new ExternalSorter(pageSize, numberOfPages));
 
 	while (stream.hasNext())
@@ -360,10 +348,6 @@ void BulkLoader::bulkLoadUsingSTR(
 
 	while (true)
 	{
-		#ifndef NDEBUG
-		std::cerr << "RTree::BulkLoader: Building level " << level << std::endl;
-		#endif
-
 		pTree->m_stats.m_nodesInLevel.push_back(0);
 
         std::shared_ptr<ExternalSorter> es2 = std::shared_ptr<ExternalSorter>(new ExternalSorter(pageSize, numberOfPages));
