@@ -40,12 +40,12 @@
 using namespace SpatialIndex::RTree;
 using namespace SpatialIndex;
 
-SpatialIndex::RTree::Data::Data(uint32_t len, byte* pData, Region& r, id_type id)
+SpatialIndex::RTree::Data::Data(uint32_t len, uint8_t* pData, Region& r, id_type id)
 	: m_id(id), m_region(r), m_pData(nullptr), m_dataLength(len)
 {
 	if (m_dataLength > 0)
 	{
-		m_pData = new byte[m_dataLength];
+		m_pData = new uint8_t[m_dataLength];
 		memcpy(m_pData, pData, m_dataLength);
 	}
 }
@@ -70,14 +70,14 @@ void SpatialIndex::RTree::Data::getShape(IShape** out) const
 	*out = new Region(m_region);
 }
 
-void SpatialIndex::RTree::Data::getData(uint32_t& len, byte** data) const
+void SpatialIndex::RTree::Data::getData(uint32_t& len, uint8_t** data) const
 {
 	len = m_dataLength;
 	*data = nullptr;
 
 	if (m_dataLength > 0)
 	{
-		*data = new byte[m_dataLength];
+		*data = new uint8_t[m_dataLength];
 		memcpy(*data, m_pData, m_dataLength);
 	}
 }
@@ -91,7 +91,7 @@ uint32_t SpatialIndex::RTree::Data::getByteArraySize()
 		m_region.getByteArraySize();
 }
 
-void SpatialIndex::RTree::Data::loadFromByteArray(const byte* ptr)
+void SpatialIndex::RTree::Data::loadFromByteArray(const uint8_t* ptr)
 {
 	memcpy(&m_id, ptr, sizeof(id_type));
 	ptr += sizeof(id_type);
@@ -104,7 +104,7 @@ void SpatialIndex::RTree::Data::loadFromByteArray(const byte* ptr)
 
 	if (m_dataLength > 0)
 	{
-		m_pData = new byte[m_dataLength];
+		m_pData = new uint8_t[m_dataLength];
 		memcpy(m_pData, ptr, m_dataLength);
 		ptr += m_dataLength;
 	}
@@ -112,17 +112,17 @@ void SpatialIndex::RTree::Data::loadFromByteArray(const byte* ptr)
 	m_region.loadFromByteArray(ptr);
 }
 
-void SpatialIndex::RTree::Data::storeToByteArray(byte** data, uint32_t& len)
+void SpatialIndex::RTree::Data::storeToByteArray(uint8_t** data, uint32_t& len)
 {
 	// it is thread safe this way.
 	uint32_t regionsize;
-	byte* regiondata = nullptr;
+	uint8_t* regiondata = nullptr;
 	m_region.storeToByteArray(&regiondata, regionsize);
 
 	len = sizeof(id_type) + sizeof(uint32_t) + m_dataLength + regionsize;
 
-	*data = new byte[len];
-	byte* ptr = *data;
+	*data = new uint8_t[len];
+	uint8_t* ptr = *data;
 
 	memcpy(ptr, &m_id, sizeof(id_type));
 	ptr += sizeof(id_type);
@@ -208,7 +208,7 @@ SpatialIndex::ISpatialIndex* SpatialIndex::RTree::createAndBulkLoadNewRTree(
 	switch (m)
 	{
 	case BLM_STR:
-		bl.bulkLoadUsingSTR(static_cast<RTree*>(tree), stream, bindex, bleaf, 10000, 100);
+	    bl.bulkLoadUsingSTR(static_cast<RTree*>(tree), stream, bindex, bleaf, 10000, 100);
 		break;
 	default:
 		throw Tools::IllegalArgumentException("createAndBulkLoadNewRTree: Unknown bulk load method.");
@@ -226,13 +226,13 @@ SpatialIndex::ISpatialIndex* SpatialIndex::RTree::createAndBulkLoadNewRTree(
 	id_type& indexIdentifier)
 {
 	Tools::Variant var;
-	RTreeVariant rv(RV_LINEAR);
-	double fillFactor(0.0);
-	uint32_t indexCapacity(0);
-	uint32_t leafCapacity(0);
-	uint32_t dimension(0);
-	uint32_t pageSize(0);
-	uint32_t numberOfPages(0);
+	RTreeVariant rv(RV_RSTAR);
+	double fillFactor(0.7);
+	uint32_t indexCapacity(100);
+	uint32_t leafCapacity(100);
+	uint32_t dimension(2);
+	uint32_t pageSize(10000);
+	uint32_t numberOfPages(100);
 
 	// tree variant
 	var = ps.getProperty("TreeVariant");
@@ -401,7 +401,7 @@ SpatialIndex::RTree::RTree::~RTree()
 // ISpatialIndex interface
 //
 
-void SpatialIndex::RTree::RTree::insertData(uint32_t len, const byte* pData, const IShape& shape, id_type id)
+void SpatialIndex::RTree::RTree::insertData(uint32_t len, const uint8_t* pData, const IShape& shape, id_type id)
 {
 	if (shape.getDimension() != m_dimension) throw Tools::IllegalArgumentException("insertData: Shape has the wrong number of dimensions.");
 
@@ -409,11 +409,11 @@ void SpatialIndex::RTree::RTree::insertData(uint32_t len, const byte* pData, con
 	RegionPtr mbr = m_regionPool.acquire();
 	shape.getMBR(*mbr);
 
-	byte* buffer = nullptr;
+	uint8_t* buffer = nullptr;
 
 	if (len > 0)
 	{
-		buffer = new byte[len];
+		buffer = new uint8_t[len];
 		memcpy(buffer, pData, len);
 	}
 
@@ -459,7 +459,7 @@ void SpatialIndex::RTree::RTree::internalNodesQuery(const IShape& query, IVisito
 				uint64_t *obj = new uint64_t[nObj];
 				std::copy(vId.GetResults().begin(), vId.GetResults().end(), obj);
 
-				Data data = Data(sizeof(uint64_t) * nObj, (byte *) obj, n->m_nodeMBR, n->getIdentifier());
+				Data data = Data((uint32_t)(sizeof(uint64_t) * nObj), (uint8_t *) obj, n->m_nodeMBR, n->getIdentifier());
 				v.visitData(data);
 				++(m_stats.m_u64QueryResults);
 			}
@@ -471,7 +471,7 @@ void SpatialIndex::RTree::RTree::internalNodesQuery(const IShape& query, IVisito
 					{
 						if(query.containsShape(*(n->m_ptrMBR[cChild])))
 						{
-							Data data = Data(sizeof(id_type), (byte *) &n->m_pIdentifier[cChild], *(n->m_ptrMBR[cChild]), n->getIdentifier());
+							Data data = Data(sizeof(id_type), (uint8_t *) &n->m_pIdentifier[cChild], *(n->m_ptrMBR[cChild]), n->getIdentifier());
 							v.visitData(data);
 							++(m_stats.m_u64QueryResults);
 						}
@@ -562,73 +562,87 @@ void SpatialIndex::RTree::RTree::pointLocationQuery(const Point& query, IVisitor
 	rangeQuery(IntersectionQuery, r, v);
 }
 
-void SpatialIndex::RTree::RTree::nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v, INearestNeighborComparator& nnc)
-{
-	if (query.getDimension() != m_dimension) throw Tools::IllegalArgumentException("nearestNeighborQuery: Shape has the wrong number of dimensions.");
-
-	std::priority_queue<NNEntry*, std::vector<NNEntry*>, NNEntry::ascending> queue;
-
-	queue.push(new NNEntry(m_rootID, nullptr, 0.0));
-
-	uint32_t count = 0;
-	double knearest = 0.0;
-
-	while (! queue.empty())
-	{
-		NNEntry* pFirst = queue.top();
-
-		// report all nearest neighbors with equal greatest distances.
-		// (neighbors can be more than k, if many happen to have the same greatest distance).
-		if (count >= k && pFirst->m_minDist > knearest)	break;
-
-		queue.pop();
-
-		if (pFirst->m_pEntry == nullptr)
-		{
-			// n is a leaf or an index.
-			NodePtr n = readNode(pFirst->m_id);
-			v.visitNode(*n);
-
-			for (uint32_t cChild = 0; cChild < n->m_children; ++cChild)
-			{
-				if (n->m_level == 0)
-				{
-					Data* e = new Data(n->m_pDataLength[cChild], n->m_pData[cChild], *(n->m_ptrMBR[cChild]), n->m_pIdentifier[cChild]);
-					// we need to compare the query with the actual data entry here, so we call the
-					// appropriate getMinimumDistance method of NearestNeighborComparator.
-					queue.push(new NNEntry(n->m_pIdentifier[cChild], e, nnc.getMinimumDistance(query, *e)));
-				}
-				else
-				{
-					queue.push(new NNEntry(n->m_pIdentifier[cChild], nullptr, nnc.getMinimumDistance(query, *(n->m_ptrMBR[cChild]))));
-				}
-			}
-		}
-		else
-		{
-			v.visitData(*(static_cast<IData*>(pFirst->m_pEntry)));
-			++(m_stats.m_u64QueryResults);
-			++count;
-			knearest = pFirst->m_minDist;
-			delete pFirst->m_pEntry;
-		}
-
-		delete pFirst;
-	}
-
-	while (! queue.empty())
-	{
-		NNEntry* e = queue.top(); queue.pop();
-		if (e->m_pEntry != nullptr) delete e->m_pEntry;
-		delete e;
-	}
+template <class T, class S, class C>
+    S& Container(std::priority_queue<T, S, C>& q) {
+        struct HackedQueue : private std::priority_queue<T, S, C> {
+            static S& Container(std::priority_queue<T, S, C>& q) {
+                return q.*&HackedQueue::c;
+            }
+        };
+    return HackedQueue::Container(q);
 }
 
-void SpatialIndex::RTree::RTree::nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v)
+double SpatialIndex::RTree::RTree::nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v, INearestNeighborComparator& nnc, double max_dist)
+{
+    if (query.getDimension() != m_dimension) throw Tools::IllegalArgumentException("nearestNeighborQuery: Shape has the wrong number of dimensions.");
+
+    auto ascending = [](const NNEntry& lhs, const NNEntry& rhs) { return lhs.m_minDist > rhs.m_minDist;  };
+    std::priority_queue<NNEntry, std::vector<NNEntry>, decltype(ascending)> queue(ascending);
+
+    // Extract the container from the queue
+    std::vector<NNEntry>& queue_c = Container(queue);
+    queue_c.reserve(64);
+
+    queue.push(NNEntry(m_rootID, nullptr, 0.0));
+
+    uint32_t count = 0;
+    double knearest = 0.0;
+
+    while (! queue.empty())
+    {
+        NNEntry pFirst = queue.top();
+
+        if (max_dist && pFirst.m_minDist > max_dist) break;
+
+        // report all nearest neighbors with equal greatest distances.
+        // (neighbors can be more than k, if many happen to have the same greatest distance).
+        if (count >= k && pFirst.m_minDist > knearest) break;
+
+        queue.pop();
+
+        if (pFirst.m_pEntry == nullptr)
+        {
+            // n is a leaf or an index.
+            NodePtr n = readNode(pFirst.m_id);
+            v.visitNode(*n);
+
+            for (uint32_t cChild = 0; cChild < n->m_children; ++cChild)
+            {
+                if (n->m_level == 0)
+                {
+                    Data* e = new Data(n->m_pDataLength[cChild], n->m_pData[cChild], *(n->m_ptrMBR[cChild]), n->m_pIdentifier[cChild]);
+                    // we need to compare the query with the actual data entry here, so we call the
+                    // appropriate getMinimumDistance method of NearestNeighborComparator.
+                    queue.push(NNEntry(n->m_pIdentifier[cChild], e, nnc.getMinimumDistance(query, e->m_region)));
+                }
+                else
+                {
+                    queue.push(NNEntry(n->m_pIdentifier[cChild], nullptr, nnc.getMinimumDistance(query, *(n->m_ptrMBR[cChild]))));
+                }
+            }
+        }
+        else
+        {
+            v.visitData(*(static_cast<IData*>(pFirst.m_pEntry)));
+            ++(m_stats.m_u64QueryResults);
+            ++count;
+            knearest = pFirst.m_minDist;
+            delete pFirst.m_pEntry;
+        }
+    }
+
+    for (auto& e : queue_c)
+        if (e.m_pEntry)
+            delete e.m_pEntry;
+
+    return knearest;
+}
+
+double SpatialIndex::RTree::RTree::nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v, double max_dist)
 {
 	if (query.getDimension() != m_dimension) throw Tools::IllegalArgumentException("nearestNeighborQuery: Shape has the wrong number of dimensions.");
 	NNComparator nnc;
-	nearestNeighborQuery(k, query, v, nnc);
+	return nearestNeighborQuery(k, query, v, nnc, max_dist);
 }
 
 
@@ -1142,8 +1156,8 @@ void SpatialIndex::RTree::RTree::storeHeader()
 		sizeof(uint32_t) +						// m_stats.m_treeHeight
 		m_stats.m_u32TreeHeight * sizeof(uint32_t);	// m_stats.m_nodesInLevel
 
-	byte* header = new byte[headerSize];
-	byte* ptr = header;
+	uint8_t* header = new uint8_t[headerSize];
+	uint8_t* ptr = header;
 
 	memcpy(ptr, &m_rootID, sizeof(id_type));
 	ptr += sizeof(id_type);
@@ -1187,10 +1201,10 @@ void SpatialIndex::RTree::RTree::storeHeader()
 void SpatialIndex::RTree::RTree::loadHeader()
 {
 	uint32_t headerSize;
-	byte* header = nullptr;
+	uint8_t* header = nullptr;
 	m_pStorageManager->loadByteArray(m_headerID, headerSize, &header);
 
-	byte* ptr = header;
+	uint8_t* ptr = header;
 
 	memcpy(&m_rootID, ptr, sizeof(id_type));
 	ptr += sizeof(id_type);
@@ -1232,18 +1246,18 @@ void SpatialIndex::RTree::RTree::loadHeader()
 	delete[] header;
 }
 
-void SpatialIndex::RTree::RTree::insertData_impl(uint32_t dataLength, byte* pData, Region& mbr, id_type id)
+void SpatialIndex::RTree::RTree::insertData_impl(uint32_t dataLength, uint8_t* pData, Region& mbr, id_type id)
 {
 	assert(mbr.getDimension() == m_dimension);
 
 	std::stack<id_type> pathBuffer;
-	byte* overflowTable = nullptr;
+	uint8_t* overflowTable = nullptr;
 
 	try
 	{
 		NodePtr root = readNode(m_rootID);
 
-		overflowTable = new byte[root->m_level];
+		overflowTable = new uint8_t[root->m_level];
 		memset(overflowTable, 0, root->m_level);
 
 		NodePtr l = root->chooseSubtree(mbr, 0, pathBuffer);
@@ -1264,7 +1278,7 @@ void SpatialIndex::RTree::RTree::insertData_impl(uint32_t dataLength, byte* pDat
 	}
 }
 
-void SpatialIndex::RTree::RTree::insertData_impl(uint32_t dataLength, byte* pData, Region& mbr, id_type id, uint32_t level, byte* overflowTable)
+void SpatialIndex::RTree::RTree::insertData_impl(uint32_t dataLength, uint8_t* pData, Region& mbr, id_type id, uint32_t level, uint8_t* overflowTable)
 {
 	assert(mbr.getDimension() == m_dimension);
 
@@ -1308,7 +1322,7 @@ bool SpatialIndex::RTree::RTree::deleteData_impl(const Region& mbr, id_type id)
 
 SpatialIndex::id_type SpatialIndex::RTree::RTree::writeNode(Node* n)
 {
-	byte* buffer;
+	uint8_t* buffer;
 	uint32_t dataLength;
 	n->storeToByteArray(&buffer, dataLength);
 
@@ -1333,18 +1347,7 @@ SpatialIndex::id_type SpatialIndex::RTree::RTree::writeNode(Node* n)
 		n->m_identifier = page;
 		++(m_stats.m_u32Nodes);
 
-#ifndef NDEBUG
-		try
-		{
-			m_stats.m_nodesInLevel[n->m_level] = m_stats.m_nodesInLevel.at(n->m_level) + 1;
-		}
-		catch(...)
-		{
-			throw Tools::IllegalStateException("writeNode: writing past the end of m_nodesInLevel.");
-		}
-#else
 		m_stats.m_nodesInLevel[n->m_level] = m_stats.m_nodesInLevel[n->m_level] + 1;
-#endif
 	}
 
 	++(m_stats.m_u64Writes);
@@ -1360,7 +1363,7 @@ SpatialIndex::id_type SpatialIndex::RTree::RTree::writeNode(Node* n)
 SpatialIndex::RTree::NodePtr SpatialIndex::RTree::RTree::readNode(id_type page)
 {
 	uint32_t dataLength;
-	byte* buffer;
+	uint8_t* buffer;
 
 	try
 	{
@@ -1562,16 +1565,5 @@ std::ostream& SpatialIndex::RTree::operator<<(std::ostream& os, const RTree& t)
 		os	<< "Utilization: " << 100 * t.m_stats.getNumberOfData() / (t.m_stats.getNumberOfNodesInLevel(0) * t.m_leafCapacity) << "%" << std::endl
 			<< t.m_stats;
 
-	#ifndef NDEBUG
-	os	<< "Leaf pool hits: " << t.m_leafPool.m_hits << std::endl
-		<< "Leaf pool misses: " << t.m_leafPool.m_misses << std::endl
-		<< "Index pool hits: " << t.m_indexPool.m_hits << std::endl
-		<< "Index pool misses: " << t.m_indexPool.m_misses << std::endl
-		<< "Region pool hits: " << t.m_regionPool.m_hits << std::endl
-		<< "Region pool misses: " << t.m_regionPool.m_misses << std::endl
-		<< "Point pool hits: " << t.m_pointPool.m_hits << std::endl
-		<< "Point pool misses: " << t.m_pointPool.m_misses << std::endl;
-	#endif
-
 	return os;
 }
