@@ -44,7 +44,7 @@ namespace SpatialIndex
 			RTree(IStorageManager&, Tools::PropertySet&);
 				// String                   Value     Description
 				// ----------------------------------------------
-				// IndexIndentifier         VT_LONG   If specified an existing index will be openened from the supplied
+				// IndexIndentifier         VT_LONG   If specified an existing index will be opened from the supplied
 				//                          storage manager with the given index id. Behaviour is unspecified
 				//                          if the index id or the storage manager are incorrect.
 				// Dimension                VT_ULONG  Dimensionality of the data that will be inserted.
@@ -74,8 +74,8 @@ namespace SpatialIndex
 			virtual void containsWhatQuery(const IShape& query, IVisitor& v) ;
 			virtual void intersectsWithQuery(const IShape& query, IVisitor& v) ;
 			virtual void pointLocationQuery(const Point& query, IVisitor& v) ;
-			virtual void nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v, INearestNeighborComparator&);
-			virtual void nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v);
+			virtual double nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v, INearestNeighborComparator&, double max_dist=0.0);
+			virtual double nearestNeighborQuery(uint32_t k, const IShape& query, IVisitor& v, double max_dist=0.0);
 			virtual void selfJoinQuery(const IShape& s, IVisitor& v) ;
 			virtual void queryStrategy(IQueryStrategy& qs) ;
 			virtual void getIndexProperties(Tools::PropertySet& out) const ;
@@ -120,7 +120,7 @@ namespace SpatialIndex
 				// for Points and Rectangles', Section 4.1]
 
 			double m_splitDistributionFactor;
-				// The R*-Tree 'm' constant, for calculating spliting distributions.
+				// The R*-Tree 'm' constant, for calculating splitting distributions.
 				// [Beckmann, Kriegel, Schneider, Seeger 'The R*-tree: An efficient and Robust Access Method
 				// for Points and Rectangles', Section 4.2]
 
@@ -150,10 +150,10 @@ namespace SpatialIndex
 			{
 			public:
 				id_type m_id;
-				IEntry* m_pEntry;
+				Data* m_pEntry;
 				double m_minDist;
 
-				NNEntry(id_type id, IEntry* e, double f) : m_id(id), m_pEntry(e), m_minDist(f) {}
+				NNEntry(id_type id, Data* e, double f) : m_id(id), m_pEntry(e), m_minDist(f) {}
 				~NNEntry() = default;
 
 			}; // NNEntry
@@ -161,12 +161,12 @@ namespace SpatialIndex
 			class NNComparator : public INearestNeighborComparator
 			{
 			public:
-				double getMinimumDistance(const IShape& query, const IShape& entry) 
+				double getMinimumDistance(const IShape& query, const IShape& entry)
 				{
 					return query.getMinimumDistance(entry);
 				}
 
-				double getMinimumDistance(const IShape& query, const IData& data) 
+				double getMinimumDistance(const IShape& query, const IData& data)
 				{
 					IShape* pS;
 					data.getShape(&pS);
