@@ -5,7 +5,7 @@
  * Copyright (c) 2004, Marios Hadjieleftheriou
  *
  * All rights reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -58,35 +58,14 @@ Region::Region(const Region& r)
 
 void Region::initialize(const double* pLow, const double* pHigh, uint32_t dimension)
 {
-	m_pLow = nullptr;
 	m_dimension = dimension;
 
-#ifndef NDEBUG
-    for (uint32_t cDim = 0; cDim < m_dimension; ++cDim)
-    {
-     if ((pLow[cDim] > pHigh[cDim]))
-     {
-         // check for infinitive region
-         if (!(pLow[cDim] == std::numeric_limits<double>::max() ||
-             pHigh[cDim] == -std::numeric_limits<double>::max() ))
-             throw Tools::IllegalArgumentException(
-                 "Region::initialize: Low point has larger coordinates than High point."
-                 " Neither point is infinity."
-             );
-     }
-    }
-#endif
+	if (m_dimension <= local_dim)
+	   m_pLow = m_local;
+	else
+	   m_pLow = new double[2*m_dimension];
 
-	try
-	{
-		m_pLow = new double[m_dimension];
-		m_pHigh = new double[m_dimension];
-	}
-	catch (...)
-	{
-		delete[] m_pLow;
-		throw;
-	}
+	m_pHigh = m_pLow + m_dimension;
 
 	memcpy(m_pLow, pLow, m_dimension * sizeof(double));
 	memcpy(m_pHigh, pHigh, m_dimension * sizeof(double));
@@ -94,8 +73,8 @@ void Region::initialize(const double* pLow, const double* pHigh, uint32_t dimens
 
 Region::~Region()
 {
-	delete[] m_pLow;
-	delete[] m_pHigh;
+    if (m_dimension > local_dim)
+        delete[] m_pLow;
 }
 
 Region& Region::operator=(const Region& r)
@@ -295,7 +274,7 @@ bool Region::touchesRegion(const Region& r) const
 		throw Tools::IllegalArgumentException(
 			"Region::touchesRegion: Regions have different number of dimensions."
 		);
-	
+
 	for (uint32_t i = 0; i < m_dimension; ++i)
 	{
 		if (
@@ -361,11 +340,11 @@ bool Region::intersectsLineSegment(const LineSegment& in) const
     // Points/LineSegment for the segment
     Point p1 = Point(in.m_pStartPoint, 2);
     Point p2 = Point(in.m_pEndPoint, 2);
-    
+
 
     //Check whether either or both the endpoints are within the region OR
     //whether any of the bounding segments of the Region intersect the segment
-    return (containsPoint(p1) || containsPoint(p2) || 
+    return (containsPoint(p1) || containsPoint(p2) ||
             in.intersectsShape(LineSegment(ll, ul)) || in.intersectsShape(LineSegment(ul, ur)) ||
             in.intersectsShape(LineSegment(ur, lr)) || in.intersectsShape(LineSegment(lr, ll)));
 
@@ -477,7 +456,7 @@ double Region::getIntersectingArea(const Region& r) const
 }
 
 /*
- * Returns the margin of a region. It is calcuated as the sum of  2^(d-1) * width, in each dimension.
+ * Returns the margin of a region. It is calculated as the sum of  2^(d-1) * width, in each dimension.
  * It is actually the sum of all edges, no matter what the dimensionality is.
 */
 double Region::getMargin() const
@@ -562,16 +541,20 @@ void Region::makeDimension(uint32_t dimension)
 {
 	if (m_dimension != dimension)
 	{
-		delete[] m_pLow;
-		delete[] m_pHigh;
+	    if (m_dimension > local_dim)
+		    delete[] m_pLow;
 
 		// remember that this is not a constructor. The object will be destructed normally if
 		// something goes wrong (bad_alloc), so we must take care not to leave the object at an intermediate state.
 		m_pLow = nullptr; m_pHigh = nullptr;
-
 		m_dimension = dimension;
-		m_pLow = new double[m_dimension];
-		m_pHigh = new double[m_dimension];
+
+		if (m_dimension <= local_dim)
+		   m_pLow = m_local;
+		else
+		   m_pLow = new double[2*m_dimension];
+
+		m_pHigh = m_pLow + m_dimension;
 	}
 }
 
