@@ -5,7 +5,7 @@
  * Copyright (c) 2002, Marios Hadjieleftheriou
  *
  * All rights reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -321,13 +321,6 @@ void Index::adjustTree(Node* n, std::stack<id_type>& pathBuffer)
 		}
 	//}
 
-#ifndef NDEBUG
-	for (uint32_t cChild = 0; cChild < m_children; ++cChild)
-	{
-		assert(m_nodeMBR.containsRegionAfterTime(m_pTree->m_currentTime, *(m_ptrMBR[cChild])) == true);
-	}
-#endif
-
 	m_pTree->writeNode(this);
 
 	if (/*! bContained && */ ! pathBuffer.empty())
@@ -382,13 +375,6 @@ void Index::adjustTree(Node* n1, Node* n2, std::stack<id_type>& pathBuffer, uint
 		}
 	//}
 
-#ifndef NDEBUG
-	for (uint32_t cChild = 0; cChild < m_children; ++cChild)
-	{
-		assert(m_nodeMBR.containsRegionAfterTime(m_pTree->m_currentTime, *(m_ptrMBR[cChild])) == true);
-	}
-#endif
-
 	// No write necessary here. insertData will write the node if needed.
 	//m_pTree->writeNode(this);
 
